@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id: grd2xyz.c 17588 2017-02-23 23:35:14Z pwessel $
  *
- *	Copyright (c) 1991-2017 by P. Wessel, W. H. F. Smith, R. Scharroo, J. Luis and F. Wobbe
+ *	Copyright (c) 1991-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *	See LICENSE.TXT file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +12,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU Lesser General Public License for more details.
  *
- *	Contact info: gmt.soest.hawaii.edu
+ *	Contact info: www.generic-mapping-tools.org
  *--------------------------------------------------------------------*/
 /*
  * Brief synopsis: grd2xyz.c reads a grid file and prints out the x,y,z values to
@@ -21,17 +20,18 @@
  *
  * Author:	Paul Wessel
  * Date:	1-JAN-2010
- * Version:	5 API
+ * Version:	6 API
  */
 
-#define THIS_MODULE_NAME	"grd2xyz"
-#define THIS_MODULE_LIB		"core"
-#define THIS_MODULE_PURPOSE	"Convert grid file to data table"
-#define THIS_MODULE_KEYS	"<G{+,>D}"
-
 #include "gmt_dev.h"
 
-#define GMT_PROG_OPTIONS "-:>RVbdfhos" GMT_OPT("H")
+#define THIS_MODULE_CLASSIC_NAME	"grd2xyz"
+#define THIS_MODULE_MODERN_NAME	"grd2xyz"
+#define THIS_MODULE_LIB		"core"
+#define THIS_MODULE_PURPOSE	"Convert grid to data table"
+#define THIS_MODULE_KEYS	"<G{+,>D}"
+#define THIS_MODULE_NEEDS	""
+#define THIS_MODULE_OPTIONS "-:>RVbdfhoqs" GMT_OPT("H")
 
 struct GRD2XYZ_CTRL {
 	struct GRD2XYZ_C {	/* -C[f|i] */
@@ -43,39 +43,42 @@ struct GRD2XYZ_CTRL {
 		bool floating;
 		double nodata;
 	} E;
-	struct GRD2XYZ_W {	/* -W[<weight>] */
+	struct GRD2XYZ_W {	/* -W[a|<weight>][+u<unit>] */
 		bool active;
+		bool area;
+		char unit;
 		double weight;
 	} W;
 	struct GMT_PARSE_Z_IO Z;
 };
 
-GMT_LOCAL void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a new control structure */
+static void *New_Ctrl (struct GMT_CTRL *GMT) {	/* Allocate and initialize a new control structure */
 	struct GRD2XYZ_CTRL *C;
-	
+
 	C = gmt_M_memory (GMT, NULL, 1, struct GRD2XYZ_CTRL);
-	
+
 	/* Initialize values whose defaults are not 0/false/NULL */
-	
+
 	C->E.nodata = -9999.0;
 	C->W.weight = 1.0;
+	C->W.unit = 'k';	/* km^2 for geo if not set */
 	C->Z.type = 'a';
 	C->Z.format[0] = 'T';	C->Z.format[1] = 'L';
-		
+
 	return (C);
 }
 
-GMT_LOCAL void Free_Ctrl (struct GMT_CTRL *GMT, struct GRD2XYZ_CTRL *C) {	/* Deallocate control structure */
+static void Free_Ctrl (struct GMT_CTRL *GMT, struct GRD2XYZ_CTRL *C) {	/* Deallocate control structure */
 	if (!C) return;
-	gmt_M_free (GMT, C);	
+	gmt_M_free (GMT, C);
 }
 
-GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
-	gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_NAME, THIS_MODULE_PURPOSE);
+static int usage (struct GMTAPI_CTRL *API, int level) {
+	const char *name = gmt_show_name_and_purpose (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_PURPOSE);
 	if (level == GMT_MODULE_PURPOSE) return (GMT_NOERROR);
-	GMT_Message (API, GMT_TIME_NONE, "usage: grd2xyz <grid> [-C[f]] [%s] [%s]\n", GMT_Rgeo_OPT, GMT_V_OPT);
-	GMT_Message (API, GMT_TIME_NONE, "\t[-W[<weight>]] [-Z[<flags>]] [%s] [%s] [%s]\n\t[%s] [%s] [%s] [%s]\n\n",
-		GMT_bo_OPT, GMT_d_OPT, GMT_f_OPT, GMT_ho_OPT, GMT_o_OPT, GMT_s_OPT, GMT_colon_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "usage: %s <grid> [-C[f|i]] [%s] [%s]\n", name, GMT_Rgeo_OPT, GMT_V_OPT);
+	GMT_Message (API, GMT_TIME_NONE, "\t[-W[a[+u<unit>]|<weight>]] [-Z[<flags>]] [%s] [%s] [%s]\n\t[%s] [%s] [%s] [%s] [%s] [%s]\n\n",
+		GMT_bo_OPT, GMT_d_OPT, GMT_f_OPT, GMT_ho_OPT, GMT_o_OPT, GMT_qo_OPT, GMT_s_OPT, GMT_colon_OPT, GMT_PAR_OPT);
 
 	if (level == GMT_SYNOPSIS) return (GMT_MODULE_SYNOPSIS);
 
@@ -85,15 +88,18 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t   Use -Ci to write grid index instead of (x,y).\n");
 	GMT_Option (API, "R,V");
 	GMT_Message (API, GMT_TIME_NONE, "\t-W Write xyzw using supplied weight (or 1 if not given) [Default is xyz].\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   Select -Wa to compute weights equal to the node areas.  If a geographic grid\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   you may append +u<unit> from %s to set area unit [k]\n", GMT_LEN_UNITS_DISPLAY);
 	GMT_Message (API, GMT_TIME_NONE, "\t-Z Set exact specification of resulting 1-column output z-table.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   If data is in row format, state if first row is at T(op) or B(ottom).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t     Then, append L or R to indicate starting point in row.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   If data is in column format, state if first columns is L(left) or R(ight).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t     Then, append T or B to indicate starting point in column.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t   To swap the byte-order of each word, append w.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Append x if gridline-registered, periodic data in x without repeating column at xmax.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Append y if gridline-registered, periodic data in y without repeating row at ymax.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   Specify one of the following data types (all binary except a):\n");
-	GMT_Message (API, GMT_TIME_NONE, "\t     a  ASCII.\n");
+	GMT_Message (API, GMT_TIME_NONE, "\t     a  ASCII (one value per record).\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t     c  int8_t, signed 1-byte character.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t     u  uint8_t, unsigned 1-byte character.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t     h  int16_t, signed short 2-byte integer.\n");
@@ -105,14 +111,12 @@ GMT_LOCAL int usage (struct GMTAPI_CTRL *API, int level) {
 	GMT_Message (API, GMT_TIME_NONE, "\t     f  4-byte floating point single precision.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t     d  8-byte floating point double precision.\n");
 	GMT_Message (API, GMT_TIME_NONE, "\t   [Default format is scanline orientation in ASCII representation: -ZTLa].\n");
-	GMT_Option (API, "bo,d,f,h,o,s,:,.");
-	
+	GMT_Option (API, "bo,d,f,h,o,qo,s,:,.");
+
 	return (GMT_MODULE_USAGE);
 }
 
-EXTERN_MSC unsigned int gmt_parse_d_option (struct GMT_CTRL *GMT, char *arg);
-
-GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRD2XYZ_CTRL *Ctrl, struct GMT_Z_IO *io, struct GMT_OPTION *options) {
+static int parse (struct GMT_CTRL *GMT, struct GRD2XYZ_CTRL *Ctrl, struct GMT_Z_IO *io, struct GMT_OPTION *options) {
 
 	/* This parses the options provided to grdcut and sets parameters in CTRL.
 	 * Any GMT common options will override values set previously by other commands.
@@ -125,14 +129,14 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRD2XYZ_CTRL *Ctrl, struct GMT
 	struct GMTAPI_CTRL *API = GMT->parent;
 
 	gmt_M_memset (io, 1, struct GMT_Z_IO);
-	
+
 	for (opt = options; opt; opt = opt->next) {	/* Process all the options given */
 
 		switch (opt->option) {
 			case '<':	/* Input files */
 				n_files++;
 				break;
-				
+
 			/* Processes program-specific parameters */
 
 			case 'C':	/* Write row,col or index instead of x,y */
@@ -144,7 +148,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRD2XYZ_CTRL *Ctrl, struct GMT
 			case 'E':	/* Old ESRI option */
 				if (gmt_M_compat_check (GMT, 4)) {
 					Ctrl->E.active = true;
-					GMT_Report (API, GMT_MSG_COMPAT, "Warning: Option -E is deprecated; use grdconvert instead.\n");
+					GMT_Report (API, GMT_MSG_COMPAT, "Option -E is deprecated; use grdconvert instead.\n");
 					if (opt->arg[0] == 'f') Ctrl->E.floating = true;
 					if (opt->arg[Ctrl->E.floating]) Ctrl->E.nodata = atof (&opt->arg[Ctrl->E.floating]);
 				}
@@ -153,12 +157,9 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRD2XYZ_CTRL *Ctrl, struct GMT
 				break;
 			case 'S':	/* Suppress/no-suppress NaNs on output */
 				if (gmt_M_compat_check (GMT, 4)) {
-					GMT_Report (API, GMT_MSG_COMPAT, "Warning: Option -S is deprecated; use -s instead.\n");
-					gmt_M_memset (GMT->current.io.io_nan_col, GMT_MAX_COLUMNS, int);
-					GMT->current.io.io_nan_col[0] = GMT_Z;	/* The default is to examine the z-column */
-					GMT->current.io.io_nan_ncols = GMT_IO_NAN_SKIP;		/* Default is that single z column */
-					GMT->current.setting.io_nan_mode = 1;	/* Plain -S */
-					if (opt->arg[0] == 'r') GMT->current.setting.io_nan_mode = GMT_IO_NAN_KEEP;	/* Old -Sr */
+					GMT_Report (API, GMT_MSG_COMPAT, "Option -S is deprecated; use -s common option instead.\n");
+					if (gmt_parse_s_option (GMT, opt->arg))
+						n_errors++;
 					GMT->common.s.active = true;
 				}
 				else
@@ -166,7 +167,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRD2XYZ_CTRL *Ctrl, struct GMT
 				break;
 			case 'N':	/* Nan-value */
 				if (gmt_M_compat_check (GMT, 4)) {	/* Honor old -N[i]<value> option */
-					GMT_Report (API, GMT_MSG_COMPAT, "Warning: Option -N is deprecated; use GMT common option -d[i|o]<nodata> instead.\n");
+					GMT_Report (API, GMT_MSG_COMPAT, "Option -N is deprecated; use GMT common option -d[i|o]<nodata> instead.\n");
 					if (opt->arg[0]) {
 						if (opt->arg[0] == 'i')	/* Simulate -di<nodata> */
 							n_errors += gmt_parse_d_option (GMT, opt->arg);
@@ -177,7 +178,7 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRD2XYZ_CTRL *Ctrl, struct GMT
 						}
 					}
 					else {
-						GMT_Report (API, GMT_MSG_NORMAL, "Syntax error -N option: Must specify value or NaN\n");
+						GMT_Report (API, GMT_MSG_ERROR, "Option -N: Must specify value or NaN\n");
 						n_errors++;
 					}
 				}
@@ -186,7 +187,15 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRD2XYZ_CTRL *Ctrl, struct GMT
 				break;
 			case 'W':	/* Add weight on output */
 				Ctrl->W.active = true;
-				if (opt->arg[0]) Ctrl->W.weight = atof (opt->arg);
+				if (opt->arg[0] == 'a') {
+					char *c = NULL;
+					Ctrl->W.area = true;
+					if ((c = strstr (opt->arg, "+u"))) {
+						Ctrl->W.unit = c[2];
+					}
+				}
+				else if (opt->arg[0])
+					Ctrl->W.weight = atof (opt->arg);
 				break;
 			case 'Z':	/* Control format */
 				Ctrl->Z.active = true;
@@ -199,11 +208,11 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRD2XYZ_CTRL *Ctrl, struct GMT
 		}
 	}
 
-	if (Ctrl->Z.active) gmt_init_z_io (GMT, Ctrl->Z.format, Ctrl->Z.repeat, Ctrl->Z.swab, Ctrl->Z.skip, Ctrl->Z.type, io);
+	if (Ctrl->Z.active) n_errors += gmt_init_z_io (GMT, Ctrl->Z.format, Ctrl->Z.repeat, Ctrl->Z.swab, Ctrl->Z.skip, Ctrl->Z.type, io);
 
-	n_errors += gmt_M_check_condition (GMT, n_files == 0, "Syntax error: Must specify at least one input file\n");
-	n_errors += gmt_M_check_condition (GMT, n_files > 1 && Ctrl->E.active, "Syntax error: -E can only handle one input file\n");
-	n_errors += gmt_M_check_condition (GMT, Ctrl->Z.active && Ctrl->E.active, "Syntax error: -E is not compatible with -Z\n");
+	n_errors += gmt_M_check_condition (GMT, n_files == 0, "Must specify at least one input file\n");
+	n_errors += gmt_M_check_condition (GMT, n_files > 1 && Ctrl->E.active, "Option -E can only handle one input file\n");
+	n_errors += gmt_M_check_condition (GMT, Ctrl->Z.active && Ctrl->E.active, "Option -E is not compatible with -Z\n");
 
 	return (n_errors ? GMT_PARSE_ERROR : GMT_NOERROR);
 }
@@ -211,18 +220,20 @@ GMT_LOCAL int parse (struct GMT_CTRL *GMT, struct GRD2XYZ_CTRL *Ctrl, struct GMT
 #define bailout(code) {gmt_M_free_options (mode); return (code);}
 #define Return(code) {Free_Ctrl (GMT, Ctrl); gmt_end_module (GMT, GMT_cpy); bailout (code);}
 
-int GMT_grd2xyz (void *V_API, int mode, void *args) {
-	bool first = true;
-	unsigned int row, col, n_output;
+EXTERN_MSC int GMT_grd2xyz (void *V_API, int mode, void *args) {
+	bool first = true, first_geo = true;
+	unsigned int row, col, n_output, w_col = 3;
 	int error = 0, write_error = 0;
-	
+
 	uint64_t ij, ij_gmt, n_total = 0, n_suppressed = 0;
 
 	char header[GMT_BUFSIZ];
 
-	double wesn[4], d_value, out[4], *x = NULL, *y = NULL;
+	double wesn[4], d_value, A_scale = 1.0, A_geo = 0.0, out[4], *x = NULL, *y = NULL;
 
-	struct GMT_GRID *G = NULL;
+	struct GMT_GRID *G = NULL, *W = NULL;
+	struct GMT_GRID_HEADER_HIDDEN *H = NULL;
+	struct GMT_RECORD *Out = NULL;
 	struct GMT_Z_IO io;
 	struct GMT_OPTION *opt = NULL;
 	struct GRD2XYZ_CTRL *Ctrl = NULL;
@@ -236,36 +247,42 @@ int GMT_grd2xyz (void *V_API, int mode, void *args) {
 	if (mode == GMT_MODULE_PURPOSE) return (usage (API, GMT_MODULE_PURPOSE));	/* Return the purpose of program */
 	options = GMT_Create_Options (API, mode, args);	if (API->error) return (API->error);	/* Set or get option list */
 
-	if (!options || options->option == GMT_OPT_USAGE) bailout (usage (API, GMT_USAGE));	/* Return the usage message */
-	if (options->option == GMT_OPT_SYNOPSIS) bailout (usage (API, GMT_SYNOPSIS));	/* Return the synopsis */
+	if ((error = gmt_report_usage (API, options, 0, usage)) != GMT_NOERROR) bailout (error);	/* Give usage if requested */
 
 	/* Parse the command-line arguments */
 
-	GMT = gmt_begin_module (API, THIS_MODULE_LIB, THIS_MODULE_NAME, &GMT_cpy); /* Save current state */
-	if (GMT_Parse_Common (API, GMT_PROG_OPTIONS, options)) Return (API->error);
+	if ((GMT = gmt_init_module (API, THIS_MODULE_LIB, THIS_MODULE_CLASSIC_NAME, THIS_MODULE_KEYS, THIS_MODULE_NEEDS, NULL, &options, &GMT_cpy)) == NULL) bailout (API->error); /* Save current state */
+	if (GMT_Parse_Common (API, THIS_MODULE_OPTIONS, options)) Return (API->error);
 	Ctrl = New_Ctrl (GMT);	/* Allocate and initialize a new control structure */
 	if ((error = parse (GMT, Ctrl, &io, options)) != 0) Return (error);
-	
+
 	/*---------------------------- This is the grd2xyz main code ----------------------------*/
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "Processing input grid(s)\n");
-	
+	GMT_Report (API, GMT_MSG_INFORMATION, "Processing input grid(s)\n");
+
 	gmt_M_memcpy (wesn, GMT->common.R.wesn, 4, double);	/* Current -R setting, if any */
 
 	if (GMT->common.b.active[GMT_OUT]) {
 		if (Ctrl->Z.active && !io.binary) {
-			GMT_Report (API, GMT_MSG_NORMAL, "Warning: -Z overrides -bo\n");
+			GMT_Report (API, GMT_MSG_ERROR, "-Z overrides -bo\n");
 			GMT->common.b.active[GMT_OUT] = false;
 		}
 		if (Ctrl->E.active && gmt_M_compat_check (GMT, 4)) {
-			GMT_Report (API, GMT_MSG_NORMAL, "Warning: -E overrides -bo\n");
+			GMT_Report (API, GMT_MSG_ERROR, "-E overrides -bo\n");
 			GMT->common.b.active[GMT_OUT] = false;
 		}
 	}
 	else if (io.binary) GMT->common.b.active[GMT_OUT] = true;
 
 	n_output = (Ctrl->Z.active) ? 1 : ((Ctrl->W.active) ? 4 : ((Ctrl->C.mode == 2) ? 2 : 3));
-	if ((error = gmt_set_cols (GMT, GMT_OUT, n_output)) != GMT_NOERROR) Return (error);
+	if (Ctrl->Z.active)
+		n_output = 1;
+	else {
+		n_output = (Ctrl->C.mode == 2) ? 2 : 3;
+		if (Ctrl->W.active) n_output++;
+		if (Ctrl->C.mode == 2) w_col = 2;
+	}
+	if ((error = GMT_Set_Columns (API, GMT_OUT, n_output, GMT_COL_FIX_NO_TEXT)) != GMT_NOERROR) Return (error);
 
 	if (GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_POINT, GMT_OUT, GMT_ADD_DEFAULT, 0, options) != GMT_NOERROR) {	/* Registers stdout, unless already set */
 		Return (API->error);
@@ -273,42 +290,47 @@ int GMT_grd2xyz (void *V_API, int mode, void *args) {
 	if (GMT_Begin_IO (API, GMT_IS_DATASET, GMT_OUT, GMT_HEADER_ON) != GMT_NOERROR) {	/* Enables data output and sets access mode */
 		Return (API->error);
 	}
-	if (GMT_Set_Geometry (API, GMT_OUT, GMT_IS_POINT != GMT_NOERROR)) {	/* Sets output geometry */
+	if (GMT_Set_Geometry (API, GMT_OUT, GMT_IS_POINT) != GMT_NOERROR) {	/* Sets output geometry */
 		Return (API->error);
 	}
 
-	out[3] = Ctrl->W.weight;
-		
-	for (opt = options; opt; opt = opt->next) {	/* Loop over arguments, skip options */ 
+	out[w_col] = Ctrl->W.weight;
+
+	for (opt = options; opt; opt = opt->next) {	/* Loop over arguments, skip options */
 
 		if (opt->option != '<') continue;	/* We are only processing input files here */
 
-		if ((G = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_GRID_HEADER_ONLY, NULL, opt->arg, NULL)) == NULL) {
+		if ((G = GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_CONTAINER_ONLY, NULL, opt->arg, NULL)) == NULL) {
 			Return (API->error);
 		}
 
-		GMT_Report (API, GMT_MSG_VERBOSE, "Working on file %s\n", G->header->name);
+		if (gmt_M_is_subset (GMT, G->header, wesn)) {	/* Subset requested; make sure wesn matches header spacing */
+			if ((error = gmt_M_err_fail (GMT, gmt_adjust_loose_wesn (GMT, wesn, G->header), "")))
+				Return (error);
+		}
 
-		if (gmt_M_is_subset (GMT, G->header, wesn))	/* Subset requested; make sure wesn matches header spacing */
-			gmt_M_err_fail (GMT, gmt_adjust_loose_wesn (GMT, wesn, G->header), "");
-
-		if (GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_GRID_DATA_ONLY, wesn, opt->arg, G) == NULL) {
+		if (GMT_Read_Data (API, GMT_IS_GRID, GMT_IS_FILE, GMT_IS_SURFACE, GMT_DATA_ONLY, wesn, opt->arg, G) == NULL) {
 			Return (API->error);	/* Get subset */
 		}
 
+		H = gmt_get_H_hidden (G->header);
+
 		n_total += G->header->nm;
 
-		gmt_M_err_fail (GMT, gmt_set_z_io (GMT, &io, G), opt->arg);
+		if ((error = gmt_M_err_fail (GMT, gmt_set_z_io (GMT, &io, G), opt->arg)))
+			Return (error);
 
 		if (Ctrl->Z.active) {	/* Write z-values only to stdout */
 			bool previous = GMT->common.b.active[GMT_OUT], rst = false;
-			int (*save) (struct GMT_CTRL *, FILE *, uint64_t, double *);
+			int (*save) (struct GMT_CTRL *, FILE *, uint64_t, double *, char *);
 			save = GMT->current.io.output;
-			
+			Out = gmt_new_record (GMT, &d_value, NULL);	/* Since we only need to worry about numerics in this module */
+
+			if (Ctrl->Z.swab) GMT_Report (API, GMT_MSG_INFORMATION, "Binary output data will be byte swapped\n");
 			GMT->current.io.output = gmt_z_output;		/* Override and use chosen output mode */
 			GMT->common.b.active[GMT_OUT] = io.binary;	/* May have to set binary as well */
 			GMT->current.setting.io_lonlat_toggle[GMT_OUT] = false;	/* Since no x,y involved here */
-			if (GMT->current.setting.io_nan_mode && GMT->current.io.io_nan_col[0] == GMT_Z) 
+			if (GMT->current.setting.io_nan_mode && GMT->current.io.io_nan_col[0] == GMT_Z)
 				{rst = true; GMT->current.io.io_nan_col[0] = GMT_X;}	/* Since we don't do xy here, only z */
 			for (ij = 0; ij < io.n_expected; ij++) {
 				ij_gmt = io.get_gmt_ij (&io, G, ij);	/* Get the corresponding grid node */
@@ -316,11 +338,12 @@ int GMT_grd2xyz (void *V_API, int mode, void *args) {
 				if ((io.x_missing && io.gmt_i == io.x_period) || (io.y_missing && io.gmt_j == 0)) continue;
 				if (GMT->common.d.active[GMT_OUT] && gmt_M_is_dnan (d_value))	/* Grid node is NaN and -d was set, so change to nan-proxy */
 					d_value = GMT->common.d.nan_proxy[GMT_OUT];
-				else if (gmt_z_input_is_nan_proxy (GMT, GMT_Z, d_value))	/* The inverse: Grid node is nan-proxy and -di was set, so change to NaN */
+				else if (gmt_input_is_nan_proxy (GMT, d_value))	/* The inverse: Grid node is nan-proxy and -di was set, so change to NaN */
 					d_value = GMT->session.d_NaN;
-				write_error = GMT_Put_Record (API, GMT_WRITE_DATA, &d_value);
+				write_error = GMT_Put_Record (API, GMT_WRITE_DATA, Out);
 				if (write_error == GMT_NOTSET) n_suppressed++;	/* Bad value caught by -s[r] */
 			}
+			gmt_M_free (GMT, Out);
 			GMT->current.io.output = save;			/* Reset pointer */
 			GMT->common.b.active[GMT_OUT] = previous;	/* Reset binary */
 			if (rst) GMT->current.io.io_nan_col[0] = GMT_Z;	/* Reset to what it was */
@@ -329,42 +352,44 @@ int GMT_grd2xyz (void *V_API, int mode, void *args) {
 			double slop;
 			char *record = NULL, item[GMT_BUFSIZ];
 			size_t n_alloc, len, rec_len;
+			struct GMT_RECORD *Out = NULL;
 			slop = 1.0 - (G->header->inc[GMT_X] / G->header->inc[GMT_Y]);
 			if (!gmt_M_is_zero (slop)) {
-				GMT_Report (API, GMT_MSG_NORMAL, "Error: x_inc must equal y_inc when writing to ESRI format\n");
+				GMT_Report (API, GMT_MSG_ERROR, "x_inc must equal y_inc when writing to ESRI format\n");
 				Return (GMT_RUNTIME_ERROR);
 			}
 			n_alloc = G->header->n_columns * 8;	/* Assume we only need 8 bytes per item (but we will allocate more if needed) */
 			record = gmt_M_memory (GMT, NULL, G->header->n_columns, char);
-			
+			Out = gmt_new_record (GMT, NULL, record);
+
 			sprintf (record, "ncols %d\nnrows %d", G->header->n_columns, G->header->n_rows);
-			GMT_Put_Record (API, GMT_WRITE_TEXT, record);	/* Write a text record */
+			GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write a text record */
 			if (G->header->registration == GMT_GRID_PIXEL_REG) {	/* Pixel format */
 				sprintf (record, "xllcorner ");
 				sprintf (item, GMT->current.setting.format_float_out, G->header->wesn[XLO]);
 				strcat  (record, item);
-				GMT_Put_Record (API, GMT_WRITE_TEXT, record);	/* Write a text record */
+				GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write a text record */
 				sprintf (record, "yllcorner ");
 				sprintf (item, GMT->current.setting.format_float_out, G->header->wesn[YLO]);
 				strcat  (record, item);
-				GMT_Put_Record (API, GMT_WRITE_TEXT, record);	/* Write a text record */
+				GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write a text record */
 			}
 			else {	/* Gridline format */
 				sprintf (record, "xllcenter ");
 				sprintf (item, GMT->current.setting.format_float_out, G->header->wesn[XLO]);
 				strcat  (record, item);
-				GMT_Put_Record (API, GMT_WRITE_TEXT, record);	/* Write a text record */
+				GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write a text record */
 				sprintf (record, "yllcenter ");
 				sprintf (item, GMT->current.setting.format_float_out, G->header->wesn[YLO]);
 				strcat  (record, item);
-				GMT_Put_Record (API, GMT_WRITE_TEXT, record);	/* Write a text record */
+				GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write a text record */
 			}
 			sprintf (record, "cellsize ");
 			sprintf (item, GMT->current.setting.format_float_out, G->header->inc[GMT_X]);
 			strcat  (record, item);
-			GMT_Put_Record (API, GMT_WRITE_TEXT, record);	/* Write a text record */
+			GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write a text record */
 			sprintf (record, "nodata_value %ld", lrint (Ctrl->E.nodata));
-			GMT_Put_Record (API, GMT_WRITE_TEXT, record);	/* Write a text record */
+			GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write a text record */
 			gmt_M_row_loop (GMT, G, row) {	/* Scanlines, starting in the north (ymax) */
 				rec_len = 0;
 				gmt_M_col_loop (GMT, G, row, col, ij) {
@@ -383,19 +408,45 @@ int GMT_grd2xyz (void *V_API, int mode, void *args) {
 					rec_len += len;
 					if (col < (G->header->n_columns-1)) { strcat (record, " "); rec_len++;}
 				}
-				GMT_Put_Record (API, GMT_WRITE_TEXT, record);	/* Write a whole y line */
+				GMT_Put_Record (API, GMT_WRITE_DATA, Out);	/* Write a whole y line */
 			}
 			gmt_M_free (GMT, record);
+			gmt_M_free (GMT, Out);
 		}
-		else {	/* Regular x,y,z[,w] output */
+		else {	/* Regular x,y,z[,w], col,row,z[,w] or index,z[,w] output */
 			if (first && GMT_Init_IO (API, GMT_IS_DATASET, GMT_IS_POINT, GMT_OUT, GMT_ADD_STDIO_IF_NONE, 0, options) != GMT_NOERROR) {	/* Establishes data output */
 				Return (API->error);
 			}
 
+			if (Ctrl->W.area) {	/* calculate area per node */
+				W = gmt_duplicate_grid (GMT, G, GMT_DUPLICATE_ALLOC);
+				gmt_get_cellarea (GMT, W);
+				if (gmt_M_is_geographic (GMT, GMT_IN)) {	/* Need geographic area scaling  */
+					if (first_geo) {	/* Initialize the distance unit machinery once */
+						if (gmt_init_distaz (GMT, Ctrl->W.unit, gmt_M_sph_mode (GMT), GMT_MAP_DIST) == GMT_NOT_A_VALID_TYPE) {
+							gmt_free_grid (GMT, &W, true);
+							Return (GMT_NOT_A_VALID_TYPE);
+						}
+						first_geo = false;
+						if (GMT->current.map.dist[GMT_MAP_DIST].arc)	/* Wants a squared steradian-type area measure, so undo km^2 first by dividing by r^2, then scale to new arc unit^2 */
+							A_geo = pow (1000.0 * GMT->current.map.dist[GMT_MAP_DIST].scale / GMT->current.proj.mean_radius, 2.0);
+						else
+							A_geo = pow (1000.0 * GMT->current.map.dist[GMT_MAP_DIST].scale, 2.0);	/* Get final measure unit for area after converting back to m^2 first */
+					}
+					A_scale = A_geo;
+				}
+				else	/* Back to Cartesian grids */
+					A_scale = 1.0;
+			}
+
 			/* Compute grid node positions once only */
 
-			x = gmt_grd_coord (GMT, G->header, GMT_X);
-			y = gmt_grd_coord (GMT, G->header, GMT_Y);
+			if (H->var_spacing[GMT_X]) GMT_Report (API, GMT_MSG_WARNING, "Grid %s has non-equidistant x-coordinates (see grd2xyz docs for discussion)\n", opt->arg);
+			if (H->var_spacing[GMT_Y]) GMT_Report (API, GMT_MSG_WARNING, "Grid %s has non-equidistant y-coordinates (see grd2xyz docs for discussion)\n", opt->arg);
+
+			x = (G->x) ? G->x : gmt_grd_coord (GMT, G->header, GMT_X);
+			y = (G->y) ? G->y : gmt_grd_coord (GMT, G->header, GMT_Y);
+			Out = gmt_new_record (GMT, out, NULL);	/* Since we only need to worry about numerics in this module */
 			if (Ctrl->C.active) {	/* Replace x,y with col,row */
 				if (Ctrl->C.mode < 2) {
 					gmt_M_row_loop  (GMT, G, row) y[row] = row + Ctrl->C.mode;
@@ -415,9 +466,9 @@ int GMT_grd2xyz (void *V_API, int mode, void *args) {
 				}
 				if (!G->header->z_units[0]) strcpy (G->header->z_units, "z");
 				if (GMT->current.setting.io_lonlat_toggle[GMT_IN])
-					sprintf (header, "# %s%s%s%s%s", G->header->y_units, GMT->current.setting.io_col_separator, G->header->x_units, GMT->current.setting.io_col_separator, G->header->z_units);
+					sprintf (header, "%s%s%s%s%s", G->header->y_units, GMT->current.setting.io_col_separator, G->header->x_units, GMT->current.setting.io_col_separator, G->header->z_units);
 				else
-					sprintf (header, "# %s%s%s%s%s", G->header->x_units, GMT->current.setting.io_col_separator, G->header->y_units, GMT->current.setting.io_col_separator, G->header->z_units);
+					sprintf (header, "%s%s%s%s%s", G->header->x_units, GMT->current.setting.io_col_separator, G->header->y_units, GMT->current.setting.io_col_separator, G->header->z_units);
 				if (Ctrl->W.active) {
 					strcat (header, GMT->current.setting.io_col_separator);
 					strcat (header, "weight");
@@ -432,21 +483,24 @@ int GMT_grd2xyz (void *V_API, int mode, void *args) {
 					out[GMT_Y] = G->data[ij];
 					if (GMT->common.d.active[GMT_OUT] && gmt_M_is_dnan (out[GMT_Y]))	/* Input matched no-data setting, so change to NaN */
 						out[GMT_Y] = GMT->common.d.nan_proxy[GMT_OUT];
-					else if (gmt_z_input_is_nan_proxy (GMT, GMT_Z, out[GMT_Y]))
+					else if (gmt_input_is_nan_proxy (GMT, out[GMT_Y]))
 						out[GMT_Y] = GMT->session.d_NaN;
 				}
 				else {
 					out[GMT_X] = x[col];	out[GMT_Y] = y[row];	out[GMT_Z] = G->data[ij];
 					if (GMT->common.d.active[GMT_OUT] && gmt_M_is_dnan (out[GMT_Z]))	/* Input matched no-data setting, so change to NaN */
 						out[GMT_Z] = GMT->common.d.nan_proxy[GMT_OUT];
-					else if (gmt_z_input_is_nan_proxy (GMT, GMT_Z, out[GMT_Z]))
+					else if (gmt_input_is_nan_proxy (GMT, out[GMT_Z]))
 						out[GMT_Z] = GMT->session.d_NaN;
 				}
-				write_error = GMT_Put_Record (API, GMT_WRITE_DATA, out);		/* Write this to output */
+				if (Ctrl->W.area) out[w_col] = W->data[ij] * A_scale;	/* Converts area from km^2 to user-selected unit (if active) */
+				write_error = GMT_Put_Record (API, GMT_WRITE_DATA, Out);		/* Write this to output */
 				if (write_error == GMT_NOTSET) n_suppressed++;	/* Bad value caught by -s[r] */
 			}
-			gmt_M_free (GMT, x);
-			gmt_M_free (GMT, y);
+			if (G->x == NULL) gmt_M_free (GMT, x);
+			if (G->y == NULL) gmt_M_free (GMT, y);
+			gmt_M_free (GMT, Out);
+			if (W) gmt_free_grid (GMT, &W, true);
 		}
 	}
 
@@ -454,12 +508,12 @@ int GMT_grd2xyz (void *V_API, int mode, void *args) {
 		Return (API->error);
 	}
 
-	GMT_Report (API, GMT_MSG_VERBOSE, "%" PRIu64 " values extracted\n", n_total - n_suppressed);
+	GMT_Report (API, GMT_MSG_INFORMATION, "%" PRIu64 " values extracted\n", n_total - n_suppressed);
 	if (n_suppressed) {
-		if (GMT->current.setting.io_nan_mode == GMT_IO_NAN_KEEP)
-			GMT_Report (API, GMT_MSG_VERBOSE, "%" PRIu64 " finite values suppressed\n", n_suppressed);
+		if (GMT->current.setting.io_nan_mode & GMT_IO_NAN_KEEP)
+			GMT_Report (API, GMT_MSG_INFORMATION, "%" PRIu64 " finite values suppressed\n", n_suppressed);
 		else
-			GMT_Report (API, GMT_MSG_VERBOSE, "%" PRIu64" NaN values suppressed\n", n_suppressed);
+			GMT_Report (API, GMT_MSG_INFORMATION, "%" PRIu64" NaN values suppressed\n", n_suppressed);
 	}
 
 	Return (GMT_NOERROR);
