@@ -1,7 +1,6 @@
 /*-------------------------------------------------------------------------
- *	$Id: mgd77.h 17496 2017-01-28 23:56:41Z pwessel $
- * 
- *    Copyright (c) 2005-2017 by P. Wessel
+ *
+ *    Copyright (c) 2005-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *    See README file for copying and redistribution conditions.
  *
  *  File:	mgd77.h
@@ -10,10 +9,10 @@
  *
  *  Authors:    Paul Wessel, Primary Investigator, SOEST, U. of Hawaii
  *		Michael Chandler, Affiliate Researcher, SOEST, U. of Hawaii
- *		
+ *
  *  Version:	1.4
  *  Revised:	15-MAR-2006
- * 
+ *
  *-------------------------------------------------------------------------*/
 
 /*!
@@ -210,7 +209,7 @@
 
 typedef char byte;	/* Used to indicate 1-byte long integer */
 typedef char* Text;	/* Used to indicate character strings */
- 
+
 /* The MGD77 File format contains a header section consisting a set of 24 records
  * of length 80 characters each.  This information can be read and stored internally
  * in the structure MGD77_HEADER.  The two i/o functions MGD77_read_header and
@@ -265,7 +264,7 @@ struct MGD77_META {	/* Information about a cruise as derived from navigation dat
 	double G1980_1930;	/* Average difference between 1980 and 1930 gravity reference field for this cruise */
 };
 
-struct MGD77_HEADER {	
+struct MGD77_HEADER {
 	struct MGD77_HEADER_PARAMS *mgd77[2];		/* See MGD-77 Documentation from NGDC for details; [0] is ORIG, [1] is REVISED */
 	struct MGD77_META meta;				/* Holds some meta-data derived directly from data records */
 	char *author;					/* Name of author of last creation/modification */
@@ -420,7 +419,7 @@ struct MGD77_CONTROL {
 	/* Programs that need to write out MGD77 data columns in a certain order will need
 	 * to declare this structure and use the MGD77_Init function to get going
 	 */
-	
+
 	/* File path information */
 	char *MGD77_HOME;				/* Directory where paths are stored [$GMT->session.SHAREDIR/mgd77] */
 	char **MGD77_datadir;				/* Directories where MGD77 data may live */
@@ -428,7 +427,7 @@ struct MGD77_CONTROL {
 	unsigned int n_MGD77_paths;			/* Number of such directories */
 	char user[MGD77_COL_ABBREV_LEN];		/* Current user id */
 	char NGDC_id[MGD77_COL_ABBREV_LEN];		/* Current NGDC file tag id */
-	char path[GMT_BUFSIZ];				/* Full path to current file */
+	char path[PATH_MAX];				/* Full path to current file */
 	FILE *fp;					/* File pointer to current open file (not used by MGD77+) */
 	unsigned int verbose_level;			/* 0 = none, 1 = warnings, 2 = errors (combined 3 for both) */
 	unsigned int verbose_dest;			/* 1 = to stdout, 2 = to stderr */
@@ -479,7 +478,7 @@ struct MGD77_CORRECTION {	/* Holds parameters for one term of a correction for o
 	double scale;		/* Scale to apply to (value - origin) */
 	double power;		/* Power we should raise the argument to [1] */
 	double (*modifier) (double);	/* Pointer to function that will modify argument */
-	
+
 	struct MGD77_CORRECTION *next;
 };
 
@@ -508,6 +507,7 @@ EXTERN_MSC int MGD77_Write_Data (struct GMT_CTRL *GMT, char *file, struct MGD77_
 EXTERN_MSC int MGD77_Read_Data_Record (struct GMT_CTRL *GMT, struct MGD77_CONTROL *F, struct MGD77_HEADER *H, double dvals[], char *tvals[]);	/* Read a single data record (selected columns only) */
 EXTERN_MSC int MGD77_Write_Data_Record (struct GMT_CTRL *GMT, struct MGD77_CONTROL *F, struct MGD77_HEADER *H, double dvals[], char *tvals[]);	/* Write a single data record (selected columns only) */
 EXTERN_MSC void MGD77_Free_Dataset (struct GMT_CTRL *GMT, struct MGD77_DATASET **S);								/* Free memory allocated by MGD77_Read_File/MGD77_Read_Data */
+EXTERN_MSC int MGD77_Verify_Columns (struct GMT_CTRL *GMT, char *string);				/* Verify if the -F option only contains valid column names */
 EXTERN_MSC int MGD77_Select_Columns (struct GMT_CTRL *GMT, char *string, struct MGD77_CONTROL *F, unsigned int option);				/* Decode the -F option specifying the desired columns */
 EXTERN_MSC int MGD77_Get_Column (struct GMT_CTRL *GMT, char *word, struct MGD77_CONTROL *F);							/* Get column number from column name (or -1 if not present) */
 EXTERN_MSC int MGD77_Info_from_Abbrev (struct GMT_CTRL *GMT, char *name, struct MGD77_HEADER *H, int *set, int *item);
@@ -520,7 +520,7 @@ EXTERN_MSC void MGD77_Apply_Bitflags (struct GMT_CTRL *GMT, struct MGD77_CONTROL
 EXTERN_MSC void MGD77_Set_Unit (struct GMT_CTRL *GMT, char *dist, double *scale, int way);							/* Convert appended distance unit to a numerical scale to give meters */
 EXTERN_MSC int MGD77_nc_status (struct GMT_CTRL *GMT, int status);										/* Checks for netCDF errors and aborts with error message */
 EXTERN_MSC int MGD77_Process_Ignore (struct GMT_CTRL *GMT, char code, char *format);								/* Process the ignre-format option */
-EXTERN_MSC void MGD77_Ignore_Format (struct GMT_CTRL *GMT, int format);										/* Dissallow some formats for consideration */
+EXTERN_MSC void MGD77_Ignore_Format (struct GMT_CTRL *GMT, int format);										/* Disallow some formats for consideration */
 EXTERN_MSC int MGD77_Select_Format (struct GMT_CTRL *GMT, int format);										/* Only select a specific format for consideration */
 EXTERN_MSC struct MGD77_DATASET *MGD77_Create_Dataset (struct GMT_CTRL *GMT);									/* Create an empty data set structure */
 EXTERN_MSC int MGD77_Prep_Header_cdf (struct GMT_CTRL *GMT, struct MGD77_CONTROL *F, struct MGD77_DATASET *S);					/* Prepare header before we write */
@@ -566,7 +566,7 @@ EXTERN_MSC double MGD77_rdc2dt (struct GMT_CTRL *GMT, struct MGD77_CONTROL *F, i
 EXTERN_MSC void MGD77_gcal_from_dt (struct GMT_CTRL *GMT, struct MGD77_CONTROL *F, double t, struct GMT_GCAL *cal);
 EXTERN_MSC double MGD77_Eotvos (struct GMT_CTRL *GMT, double lat, double velocity, double heading);
 
-#ifdef USE_CM4 
+#ifdef USE_CM4
 EXTERN_MSC double MGD77_Calc_CM4 (struct GMT_CTRL *GMT, struct MGD77_CONTROL *F, double time, double lon, double lat, bool calc_date, struct MGD77_CM4 *CM4);
 EXTERN_MSC double MGD77_Recalc_Mag_Anomaly_CM4 (struct GMT_CTRL *GMT, struct MGD77_CONTROL *F, double time, double lon, double lat, double obs, bool calc_date, struct MGD77_CM4 *CM4);
 EXTERN_MSC void MGD77_CM4_end (struct GMT_CTRL *GMT, struct MGD77_CM4 *CM4);
