@@ -1,7 +1,6 @@
 /*--------------------------------------------------------------------
- *	$Id: postscriptlight.h 18068 2017-04-29 21:40:52Z pwessel $
  *
- *	Copyright (c) 2009-2017 by P. Wessel and R. Scharroo
+ *	Copyright (c) 2009-2021 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
  *
  *	This program is free software; you can redistribute it and/or modify
  *	it under the terms of the GNU Lesser General Public License as published by
@@ -20,8 +19,8 @@
  *			   pwessel@hawaii.edu
  *		Remko Scharroo, Altimetrics
  *			   remko@altimetrics.com
- * Version:	5.3 [64-bit enabled API edition, decoupled from GMT]
- * Date:	13-NOV-2015
+ * Version:	6.0 [64-bit enabled API edition, decoupled from GMT]
+ * Date:	13-OCT-2017
  */
 
 /*!
@@ -46,10 +45,13 @@ extern "C" {
 
 #include <stdio.h>
 
+#define PSL_MaxOpStack_Size	300000	/* As of GhostSCript 9.50; see declaration in gs_init.ps */
+
 /* Number of PostScript points in one inch */
 
 #define PSL_POINTS_PER_INCH	72.0
 #define PSL_DOTS_PER_INCH	1200.0	/* Effective dots per inch resolution */
+#define PSL_DOTS_PER_INCH_PATTERN	300.0	/* Effective dots per inch resolution for a bitmap pattern -Gp */
 #define PSL_ALL_CLIP		INT_MAX	/* Terminates all clipping */
 
 /* PSL codes for geometric symbols as expected by PSL_plotsymbol */
@@ -58,8 +60,8 @@ extern "C" {
 #define PSL_CIRCLE		((int)'c')
 #define PSL_DIAMOND		((int)'d')
 #define PSL_ELLIPSE		((int)'e')
-#define PSL_HEXAGON		((int)'h')
 #define PSL_OCTAGON		((int)'g')
+#define PSL_HEXAGON		((int)'h')
 #define PSL_INVTRIANGLE		((int)'i')
 #define PSL_ROTRECT		((int)'j')
 #define PSL_MARC		((int)'m')
@@ -130,6 +132,7 @@ enum PSL_enum_const {PSL_CM	= 0,
 	PSL_FINALIZE		= 1,
 	PSL_OVERLAY		= 1,
 	PSL_MEMORY		= 2,
+	PSL_CHANGESET		= 4,
 	PSL_MEM_ALLOC		= 2097152, /* 2 Mb of memory */
 	PSL_INIT		= 0,
 	PSL_LANDSCAPE		= 0,
@@ -140,6 +143,7 @@ enum PSL_enum_const {PSL_CM	= 0,
 	PSL_RLE			= 1,
 	PSL_LZW			= 2,
 	PSL_DEFLATE		= 3,
+	PSL_EO_CLIP		= 4,
 	PSL_NO			= 0,
 	PSL_YES			= 1,
 	PSL_FWD			= 0,
@@ -148,10 +152,10 @@ enum PSL_enum_const {PSL_CM	= 0,
 	PSL_LINEAR		= 0,
 	PSL_BEZIER		= 1,
 	PSL_MAX_EPS_FONTS	= 6,
-	PSL_MAX_DIMS		= 12,		/* Max number of dim arguments to PSL_plot_symbol */
+	PSL_MAX_DIMS		= 13,		/* Max number of dim arguments to PSL_plot_symbol */
 	PSL_N_PATTERNS		= 91,		/* Current number of predefined patterns + 1, # 91 is user-supplied */
-	PSL_NAME_LEN		= 32,		/* Max length of font names */
-	PSL_BUFSIZ		= 256U};
+	PSL_FONTNAME_LEN	= 64,		/* Max length of font names */
+	PSL_BUFSIZ		= 4096U};
 
 /* PSL codes for pen movements (used by PSL_plotpoint, PSL_plotline, PSL_plotarc) */
 
@@ -159,7 +163,8 @@ enum PSL_enum_move {PSL_DRAW	= 0,
 	PSL_MOVE		= 1,
 	PSL_STROKE		= 2,
 	PSL_REL			= 4,
-	PSL_CLOSE		= 8};
+	PSL_CLOSE		= 8,
+	PSL_CLIP		= 16};
 
 /* PSL codes for text and paragraph justification */
 
@@ -202,17 +207,30 @@ enum PSL_enum_txt {PSL_TXT_INIT	= 1,
 	PSL_TXT_ROUND		= 32,
 	PSL_TXT_CURVED		= 64,
 	PSL_TXT_FILLBOX		= 128,
-	PSL_TXT_DRAWBOX		= 256};
+	PSL_TXT_DRAWBOX		= 256,
+	PSL_TXT_FILLPEN		= 512,
+	PSL_TXT_PENFILL		= 1024};
+
+/* PSL codes for text hyphen substitution (PSL_settextmode) */
+
+enum PSL_enum_txtmode {PSL_TXTMODE_HYPHEN	= 0,
+	PSL_TXTMODE_MINUS			= 1};
 
 /* Verbosity levels */
 
-enum PSL_enum_verbose {PSL_MSG_QUIET = 0,	/* No messages whatsoever */
-	PSL_MSG_NORMAL,		/* Default output, e.g., warnings and errors only */
-	PSL_MSG_TICTOC,		/* To print a tic-toc elapsed time message */
-	PSL_MSG_COMPAT,		/* Compatibility warnings */
-	PSL_MSG_VERBOSE,	/* Warnings level -V */
-	PSL_MSG_LONG_VERBOSE,	/* Longer verbose, -Vl in some programs */
-	PSL_MSG_DEBUG};		/* Debug messages for developers mostly */
+enum PSL_enum_verbose {
+	PSL_MSG_QUIET = 0,		/* No messages whatsoever */
+	PSL_MSG_ERROR,			/* Errors only */
+	PSL_MSG_WARNING,		/* Warnings */
+	PSL_MSG_TICTOC,			/* Tic-toc elapsed time message */
+	PSL_MSG_INFORMATION,	/* Informational messages */
+	PSL_MSG_COMPAT,			/* Compatibility warnings */
+	PSL_MSG_DEBUG,			/* Debug messages */
+	/* For API backwards compatibility only */
+	PSL_MSG_NORMAL = 1,			/* Now PSL_MSG_ERROR */
+	PSL_MSG_VERBOSE = 4,		/* Now PSL_MSG_WARNING */
+	PSL_MSG_LONG_VERBOSE = 5	/* Now PSL_MSG_INFORMATION */
+	};
 
 /* Color spaces */
 
@@ -248,7 +266,7 @@ enum PSL_enum_err {PSL_BAD_VALUE = -99,	/* Bad value */
  *--------------------------------------------------------------------*/
 
 struct PSL_FONT {	/* Definition */
-	char name[PSL_NAME_LEN];/* Name of this font */
+	char name[PSL_FONTNAME_LEN];/* Name of this font */
 	double height;		/* Height of A for unit fontsize */
 	int encoded;		/* true if we never should re-encode this font (e.g. symbols) */
 				/* This is also changed to true after we do re-encode a font */
@@ -278,6 +296,8 @@ struct PSL_CTRL {
 		double linewidth;		/* Current pen thickness			*/
 		double rgb[3][4];		/* Current stroke, fill, and fs fill rgb	*/
 		double offset;			/* Current setdash offset			*/
+		double transparency;		/* Current transparency	[deprecated]			*/
+		double transparencies[2];		/* Current transparencies				*/
 		double fontsize;		/* Current font size				*/
 		double subsupsize;		/* Fractional size of super/sub-scripts		*/
 		double scapssize;		/* Fractional size of small caps		*/
@@ -287,6 +307,8 @@ struct PSL_CTRL {
 		int nclip;			/* Clip depth 					*/
 		int font_no;			/* Current font number				*/
 		int outline;			/* Current outline				*/
+		int complete;			/* true for executing a custom PSL_plot_completion procedure once */
+		int use_minus;			/* true for replacing hyphen code with minus codes */
 	} current;
 	struct INTERNAL {	/* Variables used internally only */
 		char *SHAREDIR;			/* Pointer to path of directory with postscriptlight subdirectory */
@@ -295,6 +317,7 @@ struct PSL_CTRL {
 		char *user_image[PSL_N_PATTERNS];	/* Name of user patterns		*/
 		char origin[2];			/* 'r', 'a', 'f', 'c' depending on reference for new origin x and y coordinate */
 		double offset[2];		/* Origin offset [1/1]				*/
+		double prev_offset[2];		/* Previous Origin offset [1/1]				*/
 		double p_width;			/* Paper width in points, set in PSL_beginplot();	*/
 		double p_height;		/* Paper height in points, set in PSL_beginplot();	*/
 		double dpu;			/* PS dots per unit.  Must be set through PSL_beginplot();		*/
@@ -369,6 +392,7 @@ EXTERN_MSC int PSL_beginclipping (struct PSL_CTRL *PSL, double *x, double *y, in
 EXTERN_MSC int PSL_beginlayer (struct PSL_CTRL *PSL, int layer);
 EXTERN_MSC int PSL_beginplot (struct PSL_CTRL *PSL, FILE *fp, int orientation, int overlay, int color_mode, char origin[], double offset[], double page_size[], char *title, int font_no[]);
 EXTERN_MSC int PSL_beginsession (struct PSL_CTRL *PSL, unsigned int search, char *sharedir, char *userdir);
+EXTERN_MSC int PSL_copy (struct PSL_CTRL *PSL, const char *txt);
 EXTERN_MSC int PSL_endaxes (struct PSL_CTRL *PSL);
 EXTERN_MSC int PSL_endclipping (struct PSL_CTRL *PSL, int mode);
 EXTERN_MSC int PSL_endlayer (struct PSL_CTRL *PSL);
@@ -377,6 +401,7 @@ EXTERN_MSC int PSL_endsession (struct PSL_CTRL *PSL);
 EXTERN_MSC char * PSL_getplot (struct PSL_CTRL *PSL);
 EXTERN_MSC char * PSL_makecolor (struct PSL_CTRL *PSL, double rgb[]);
 EXTERN_MSC char * PSL_makefont (struct PSL_CTRL *PSL, double size, double rgb[]);
+EXTERN_MSC char * PSL_makefontsize (struct PSL_CTRL *PSL, double size);
 EXTERN_MSC char * PSL_makepen (struct PSL_CTRL *PSL, double linewidth, double rgb[], char *pattern, double offset);
 EXTERN_MSC int PSL_plotarc (struct PSL_CTRL *PSL, double x, double y, double radius, double az1, double az2, int type);
 EXTERN_MSC int PSL_plotaxis (struct PSL_CTRL *PSL, double annotation_int, char *label, double annotfontsize, int side);
@@ -396,13 +421,17 @@ EXTERN_MSC int PSL_plottext (struct PSL_CTRL *PSL, double x, double y, double fo
 EXTERN_MSC int PSL_plottextbox (struct PSL_CTRL *PSL, double x, double y, double fontsize, char *text, double angle, int justify, double offset[], int mode);
 EXTERN_MSC int PSL_plottextline (struct PSL_CTRL *PSL, double x[], double y[], int np[], int n_segments, void *arg1, void *arg2, char *label[], double angle[], int nlabel_per_seg[], double fontsize, int justify, double offset[], int mode);
 EXTERN_MSC int PSL_loadimage (struct PSL_CTRL *PSL, char *file, struct imageinfo *header, unsigned char **image);
+EXTERN_MSC int PSL_loadeps (struct PSL_CTRL *PSL, char *file, struct imageinfo *h, unsigned char **picture);
 EXTERN_MSC int PSL_setcolor (struct PSL_CTRL *PSL, double rgb[], int mode);
+EXTERN_MSC int PSL_setcurrentpoint (struct PSL_CTRL *PSL, double x, double y);
+EXTERN_MSC int PSL_setexec (struct PSL_CTRL *PSL, int action);
 EXTERN_MSC int PSL_setdefaults (struct PSL_CTRL *PSL, double xyscales[], double page_rgb[], char *encoding);
 EXTERN_MSC int PSL_setdash (struct PSL_CTRL *PSL, char *pattern, double offset);
 EXTERN_MSC int PSL_setfill (struct PSL_CTRL *PSL, double rgb[], int outline);
 EXTERN_MSC int PSL_setfont (struct PSL_CTRL *PSL, int font_no);
 EXTERN_MSC int PSL_setfontdims (struct PSL_CTRL *PSL, double supsub, double scaps, double sup_lc, double sup_uc, double sdown);
 EXTERN_MSC int PSL_setformat (struct PSL_CTRL *PSL, int n_decimals);
+EXTERN_MSC int PSL_setimage (struct PSL_CTRL *PSL, int image_no, char *imagefile, unsigned char *image, int image_dpi, unsigned int dim[], double f_rgb[], double b_rgb[]);
 EXTERN_MSC int PSL_setlinecap (struct PSL_CTRL *PSL, int cap);
 EXTERN_MSC int PSL_setlinejoin (struct PSL_CTRL *PSL, int join);
 EXTERN_MSC int PSL_setlinewidth (struct PSL_CTRL *PSL, double linewidth);
@@ -410,6 +439,8 @@ EXTERN_MSC int PSL_setmiterlimit (struct PSL_CTRL *PSL, int limit);
 EXTERN_MSC int PSL_setorigin (struct PSL_CTRL *PSL, double x, double y, double angle, int mode);
 EXTERN_MSC int PSL_setparagraph (struct PSL_CTRL *PSL, double line_space, double par_width, int par_just);
 EXTERN_MSC int PSL_setpattern (struct PSL_CTRL *PSL, int image_no, char *imagefile, int image_dpi, double f_rgb[], double b_rgb[]);
+EXTERN_MSC int PSL_settextmode (struct PSL_CTRL *PSL, int mode);
+EXTERN_MSC int PSL_settransparencies (struct PSL_CTRL *PSL, double *transparencies);
 EXTERN_MSC int PSL_settransparencymode (struct PSL_CTRL *PSL, const char *mode);
 EXTERN_MSC int PSL_definteger (struct PSL_CTRL *PSL, const char *param, int value);
 EXTERN_MSC int PSL_defpen (struct PSL_CTRL *PSL, const char *param, double width, char *style, double offset, double rgb[]);
@@ -421,6 +452,7 @@ EXTERN_MSC int PSL_defunits (struct PSL_CTRL *PSL, const char *param, double val
 EXTERN_MSC unsigned char *psl_gray_encode (struct PSL_CTRL *PSL, size_t *nbytes, unsigned char *input);
 EXTERN_MSC void psl_set_txt_array (struct PSL_CTRL *PSL, const char *param, char *array[], int n);
 EXTERN_MSC void psl_set_int_array (struct PSL_CTRL *PSL, const char *param, int *array, int n);
+EXTERN_MSC char *psl_prepare_text (struct PSL_CTRL *PSL, char *text);
 
 /* Used indirectly by FORTRAN wrapper PSL_free_ . */
 EXTERN_MSC int PSL_free_nonmacro (void *addr);
@@ -441,7 +473,10 @@ EXTERN_MSC int PSL_fclose (struct PSL_CTRL *C);
 EXTERN_MSC void psl_vector_v4 (struct PSL_CTRL *PSL, double x, double y, double param[], double rgb[], int outline);
 #endif
 
-/*! Macro for free that excplicitly checks for NULL pointer and sets freed pointer to NULL */
+/* Deprecated */
+EXTERN_MSC int PSL_settransparency (struct PSL_CTRL *PSL, double transparency);
+
+/*! Macro for free that explicitly checks for NULL pointer and sets freed pointer to NULL */
 #define PSL_free(ptr) (free((void *)(ptr)),(ptr)=NULL)
 
 #ifdef __cplusplus
